package javax.swing.undo;

import java.util.Enumeration;
import java.util.Vector;

public class CompoundEdit extends AbstractUndoableEdit
{
    boolean inProgress;
    protected Vector<UndoableEdit> edits;
    
    public CompoundEdit() {
        this.inProgress = true;
        this.edits = new Vector();
    }
    
    @Override
    public void undo() throws CannotUndoException {
        super.undo();
        int size = this.edits.size();
        while (size-- > 0) {
            ((UndoableEdit)this.edits.elementAt(size)).undo();
        }
    }
    
    @Override
    public void redo() throws CannotRedoException {
        super.redo();
        Enumeration<UndoableEdit> elements = this.edits.elements();
        while (elements.hasMoreElements()) {
            ((UndoableEdit)elements.nextElement()).redo();
        }
    }
    
    protected UndoableEdit lastEdit() {
        int size = this.edits.size();
        if (size > 0) {
            return (UndoableEdit)this.edits.elementAt(size - 1);
        }
        return null;
    }
    
    @Override
    public void die() {
        for (int i = this.edits.size() - 1; i >= 0; --i) {
            ((UndoableEdit)this.edits.elementAt(i)).die();
        }
        super.die();
    }
    
    @Override
    public boolean addEdit(UndoableEdit undoableEdit) {
        if (!this.inProgress) {
            return false;
        }
        UndoableEdit lastEdit = this.lastEdit();
        if (lastEdit == null) {
            this.edits.addElement(undoableEdit);
        }
        else if (!lastEdit.addEdit(undoableEdit)) {
            if (undoableEdit.replaceEdit(lastEdit)) {
                this.edits.removeElementAt(this.edits.size() - 1);
            }
            this.edits.addElement(undoableEdit);
        }
        return true;
    }
    
    public void end() {
        this.inProgress = false;
    }
    
    @Override
    public boolean canUndo() {
        return !this.isInProgress() && super.canUndo();
    }
    
    @Override
    public boolean canRedo() {
        return !this.isInProgress() && super.canRedo();
    }
    
    public boolean isInProgress() {
        return this.inProgress;
    }
    
    @Override
    public boolean isSignificant() {
        Enumeration<UndoableEdit> elements = this.edits.elements();
        while (elements.hasMoreElements()) {
            if (!((UndoableEdit)elements.nextElement()).isSignificant()) {
                continue;
            }
            return true;
        }
        return false;
    }
    
    @Override
    public String getPresentationName() {
        UndoableEdit lastEdit = this.lastEdit();
        if (lastEdit != null) {
            return lastEdit.getPresentationName();
        }
        return super.getPresentationName();
    }
    
    @Override
    public String getUndoPresentationName() {
        UndoableEdit lastEdit = this.lastEdit();
        if (lastEdit != null) {
            return lastEdit.getUndoPresentationName();
        }
        return super.getUndoPresentationName();
    }
    
    @Override
    public String getRedoPresentationName() {
        UndoableEdit lastEdit = this.lastEdit();
        if (lastEdit != null) {
            return lastEdit.getRedoPresentationName();
        }
        return super.getRedoPresentationName();
    }
    
    @Override
    public String toString() {
        return super.toString() + " inProgress: " + this.inProgress + " edits: " + this.edits;
    }
}
